/**
 * @file  DFFileDevice.h
 * @brief Schnittstelle fuer die Klasse CDFFileDevice.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFFILEDEVICE_H_
#define _DFFILEDEVICE_H_

#include "typedef.h"
#include "ErrorState.h"
#include "IntelHex.h"
#include "ATmega128.h"
#include "DFFileRawFirmware.h"
#include "DFFileDisplay.h"
#include "DFFileDeviceText.h"
#include "AES.h"

// Groesse des Verschluesselungsblocks
#define IV_DFFILEDEVICE_SIZE_CRYPT  0x200

// Fester Wert fuer Ablage, um erfolgreiche Entschluesselung sicherzustellen
#define IV_DFFILEDEVICE_CRYPT_FIX   0xDA59EF31
// Ablagewerte fuer Einzelbyte der Cryptblockwerte
#define IV_DFFILEDEVICE_CF_0        43
#define IV_DFFILEDEVICE_CF_1        71
#define IV_DFFILEDEVICE_CF_2        89
#define IV_DFFILEDEVICE_CF_3        36

#define IV_DFFILEDEVICE_SN_0        34
#define IV_DFFILEDEVICE_SN_1        17
#define IV_DFFILEDEVICE_SN_2        98
#define IV_DFFILEDEVICE_SN_3        63

/*
Halten der grundlegenden Dateiinformationen.
Versionsnummer, Quelldateinamen, Blockgroessen
 */
#define IV_DFFILEDEVICE_SIZE_HEADER       0x200 // Maximale Groesze fuer Headerdaten
// Fuer das PZE-MasterIV
#define IV_DFFILEDEVICE_VERSION_PZE       0x00010000
// Fuer das ZK-MasterIV
#define IV_DFFILEDEVICE_VERSION_ZK        0x00020000
// Fuer das BDE-MasterIV
#define IV_DFFILEDEVICE_VERSION_BDE       0x00030000
// Fuer das Mobile-MasterIV
#define IV_DFFILEDEVICE_VERSION_MOBIL     0x00040000
// Fuer das MDE-BoxIV
#define IV_DFFILEDEVICE_VERSION_MDE       0x00050000
// Fuer das AE-MasterIV
#define IV_DFFILEDEVICE_VERSION_AE        0x00060000
// Fuer das Timeboy-MasterIV
#define IV_DFFILEDEVICE_VERSION_TIMEBOY   0x00070000
// Fuer mehrere Geraetetypen (Information liegt in der Info-Struktur)
#define IV_DFFILEDEVICE_VERSION_MULTIPLE  0x00080000

#pragma pack(1)

typedef struct SDFSoftwareInfo {
	unsigned char size;			    // Strukturgroesse (gleich Version der Info)
	unsigned short device;			// Geraetetyp
									#define DEVICE_PZE                               (0x0001)
									#define DEVICE_ZK                                (0x0002)
									#define DEVICE_BDE                               (0x0004)
									#define DEVICE_MOBIL                             (0x0008)
									#define DEVICE_MDEBOX                            (0x0010)
									#define DEVICE_AE                                (0x0020)
									#define DEVICE_TIMEBOY                           (0x0040)
									#define DEVICE_USART0_TO_USART1                  (0x1000)
									#define DEVICE_FURUNO_GPS_MONITOR_BYPASS         (0x2000)
	unsigned short board;			// Hardware
									#define BOARD_10                                 (0x0001)
									#define BOARD_11                                 (0x0002)
									#define BOARD_12                                 (0x0004)
									#define BOARD_13                                 (0x0008)
									#define BOARD_14                                 (0x0010)
									#define BOARD_20                                 (0x0020)
									#define BOARD_21                                 (0x0040)
	unsigned char display;			// Anzeige
									#define DISPLAY_NONE                             (0x00)
									#define DISPLAY_LEHNER_SWAP                      (0x01)
									#define DISPLAY_LEHNER                           (0x02)
									#define DISPLAY_NEUMUELLER                       (0x04)
	unsigned char keypad;			// Tastatur
									#define KEYPAD_NONE                              (0x00)
									#define KEYPAD_PZE                               (0x01)
									#define KEYPAD_BDE                               (0x02)
									#define KEYPAD_AE                                (0x04)
									#define KEYPAD_PZE_BASIC                         (0x08)
									#define KEYPAD_TIMEBOY                           (0x10)
	unsigned char bluetooth;
									#define BLUETOOTH_NONE                           (0x00)
									#define BLUETOOTH_WRAPTHOR                       (0x01)
	unsigned char mmc;
									#define MMC_NONE                                 (0x00)
									#define MMC_MEMORY                               (0x01)
									#define MMC_DISK                                 (0x02)
									#define MMC_USBHOST                              (0x04)
	unsigned char modem;
									#define MODEM_NONE                               (0x00)
									#define MODEM_MC39                               (0x01)
									#define MODEM_MC55                               (0x02)
	unsigned char rs485;
									#define RS485_NONE                               (0x00)
									#define RS485_TIP                                (0x01)
	unsigned char tcpip;
									#define TCPIP_NONE                               (0x00)
									#define TCPIP_XPORT                              (0x01)
	unsigned char usb;
									#define USB_NONE                                 (0x00)
									#define USB_TIP                                  (0x01)
	unsigned char access;
									#define ACCESS_NONE                              (0x00)
									#define ACCESS_GIS                               (0x01)
									#define ACCESS_PHG                               (0x02)
									#define ACCESS_PROTECT                           (0x04)
	unsigned char dcf77;
									#define DCF77_NONE                               (0x00)
									#define DCF77_HKW                                (0x01)
	unsigned char docking;
									#define DOCKING_NONE                             (0x00)
									#define DOCKING_ENABLE                           (0x01)
	unsigned char fingerprint;
									#define FP_NONE                                  (0x00)
									#define FP_BIOKEY                                (0x01)
	unsigned char gps;
									#define GPS_NONE                                 (0x00)
									#define GPS_FURUNO                               (0x01)
	unsigned char iomodule;
									#define IOMODULE_NONE                            (0x00)
									#define IOMODULE_ENABLE                          (0x01)
	unsigned short transponder;
									#define TP_NONE                                  (0x0000)
									#define TP_TSR30                                 (0x0001)
									#define TP_TSR32                                 (0x0002)
									#define TP_PRIMO100                              (0x0004)
									#define TP_MIFARE                                (0x0008)
									#define TP_IBUTTON_EXT                           (0x0010)
									#define TP_SMARTRELAIS                           (0x0020)
									#define TP_NEDAP                                 (0x0040)
									#define TP_IBUTTON_INT                           (0x0080)
	unsigned char analog_in;
									#define ANALOG_IN_NONE                           (0x00)
									#define ANALOG_IN_2                              (0x03)
	unsigned char digital_in;
									#define DIGITAL_IN_NONE                          (0x00)
									#define DIGITAL_IN_2                             (0x03)
									#define DIGITAL_IN_6                             (0x3F)
									#define DIGITAL_IN_8                             (0xFF)
	unsigned char digital_out;
									#define DIGITAL_OUT_NONE                         (0x00)
									#define DIGITAL_OUT_2                            (0x03)
									#define DIGITAL_OUT_4                            (0x0F)
	unsigned char trace;
									#define TRACE_NONE                               (0x00)

// Strukturgroesse der Version 1 ist: 25Byte
} TDFSoftwareInfo;


struct SDFFileDeviceHeader
{
	unsigned long versionFile; // Dateiversion (Major 1 -> PZE Master IV, Major 2 -> Zutritt Master IV, Major 3 -> BDE Master IV)
	unsigned short sizeData;   // Groesze der Headstruktur: sizeof(SDFFileDeviceHeader), wird benoetigt fuer die Berechnung der Position der crcHead - Variable
	char timestamp[20];        // Datetime vom Build, im Format "2005-01-25 07:00:00\0"
	unsigned long crcProgram;  // 32bit CRC fuer Programm-Flash
	char versionFirmware[32];  // Firmwareversion als String, im Beispielformat: "04.00.23.03\0"
	SDFSoftwareInfo softwareInfo; // Softwareinformation zur enthaltenen Anwendungsfirmware.
	//...
	unsigned short crcHead;    // 16bit CRC fuer Head (Immer letzter Wert in der Struktur; 
                               // Berechnungsmenge: (sizeData - sizeof(unsigned short)), also ohne sich selbst!)
};
#pragma pack()

class CDFFileDevice : public CIntelHex
{
public:
	CDFFileDevice();
	virtual ~CDFFileDevice();

	void Reset();
	
	inline int GetLastErrorState() { return m_errorState; };
	inline unsigned long GetHeaderVersionFile() { return m_header->versionFile; };
	inline unsigned long GetHeaderVersionTimestamp(char *timestamp) { return sprintf(timestamp, "%s", m_header->timestamp); };
	inline unsigned long GetHeaderVersionFirmware(char *versionFirmware) { return sprintf(versionFirmware, "%s", m_header->versionFirmware); };
	inline SDFSoftwareInfo GetHeaderSoftwareInfo() { return m_header->softwareInfo; }
	inline unsigned char IsCrypt() { return m_isCrypt; };
	inline void SetSerial(unsigned long numSerial) { m_numSerial = numSerial; };
	inline unsigned long GetSerial() { return m_numSerial; };
	inline unsigned long GetSerialCrypt() 
	{ 
		unsigned long numSerial;
		numSerial = m_crypt[IV_DFFILEDEVICE_SN_0];
		numSerial |= (unsigned long)m_crypt[IV_DFFILEDEVICE_SN_1] << 8;
		numSerial |= (unsigned long)m_crypt[IV_DFFILEDEVICE_SN_2] << 16;
		numSerial |= (unsigned long)m_crypt[IV_DFFILEDEVICE_SN_3] << 24;
		return numSerial;
	}

	SDFFileDeviceHeader GetHeader(char *fileName);
	int CheckVersionFile(SDFFileDeviceHeader &header);
	unsigned long GetVersionFile(char *fileName);
	unsigned long GetVersionFirmware(char *fileName, char *versionFirmware); // max. 32 Zeichen Firmwareversion
	int Load(char *fileName);
	int Save(char *fileName, SDFFileDeviceHeader &header);
	int Build(char *fileDevice, SDFFileDeviceHeader &header, char *fileFirmware, char *fileDisplayData, char *fileDeviceText);

	SDFSoftwareInfo SWInfoStringToStruct(char *data)
	{
		SDFSoftwareInfo swInfo;
		memset(&swInfo, 0, sizeof(SDFSoftwareInfo));

		// # durch : ersetzen
		data[INTELHEX_START] = INTELHEX_START_SIGN;    // start of record
		// CRC pruefen
		if (!CheckCrc(data)) {

			return swInfo;
		}
		
		unsigned char i, count   = GetCount(data);
		// spaetere Groessenanderung vorbehalten!
		for (i=0; (i < count) && i < sizeof(SDFSoftwareInfo); i++) {
			
			((unsigned char*)&swInfo)[i] = GetData8(data, i);
		}

		return swInfo;
	}

	char *SWInfoStructToString(char *data, SDFSoftwareInfo &swInfo)
	{
		if (MakeDataLine(0, (unsigned char*)&swInfo, sizeof(SDFSoftwareInfo), data) == 0) {

			return NULL;
		}
		// : durch # ersetzen
		data[INTELHEX_START] = '#';    // start of record

		return data;
	}

	int CompareInfos(TDFHardwareInfo hwInfo, TDFSoftwareInfo swInfo);
	int GetInfos(TDFSoftwareInfo swInfo, char *info);

	unsigned long GetSizeData();
	inline CDFFileDisplay *GetFileDisplay() { return &m_fileDisplay; };
	inline CATmega128 *GetController() { return m_fileRawFirmware.GetController(); };

	// Verschluesseln
	int Encrypt(unsigned long numSerial);
	// Entschluesseln
	int Decrypt();

	CDFFileRawFirmware m_fileRawFirmware;
	char m_fileName[260];
	char m_errorText[0x100];
protected:
	int m_errorState;
	// Der Dateiheader wird ueber einen Puffer verwaltet, somit ist eine
	// vergroesserte Struktur auch von aelteren Softwarestaenden lesbar und
	// eine Pruefung des crcHead durch info:sizeData moeglich.
	unsigned char m_headBuf[IV_DFFILEDEVICE_SIZE_HEADER];
	SDFFileDeviceHeader *m_header;
	CDFFileDisplay m_fileDisplay;
	CDFFileDeviceText m_fileDeviceText;

protected:
	unsigned char m_crypt[IV_DFFILEDEVICE_SIZE_CRYPT];
	unsigned char m_isCrypt;
	unsigned long m_numSerial;
};

#endif

