/** 
* @file TimeboyStation.h
* @brief Schnittstelle fuer die Klasse TimeboyStation.
*/
//////////////////////////////////////////////////////////////////////

#ifndef _TIMEBOY_STATION_H_
#define _TIMEBOY_STATION_H_

// Basisklasse einbinden.
#include "Channel4Base.h"
#include <time.h>

//######################## TIMEBOY SLOT #######################################
// Defines Protokollbefehle
#define MB_READ		0x21		// bis zu 64 Byte lesen
#define MB_WRITE	0x22		// bis zu 64 Byte schreiben

// Defines fuer bestimmte Daten zu einem Slot
#define MB_SLOT_CHECK                  0x8000000C // Slot parameter fuer De/aktivierung der Timeboyerkennung
#define MB_SLOT_LED                    0x80001000 // Rote LED ein/aus
// Schreibe 1 Byte mit Wert 0 fuer: ein
// Schreibe 1 Byte mit Wert 255 fuer: aus
#define MB_SLOT_STATE                  0x8000E000 // Aktueller Steckzustand im Slot
#define MB_SLOT_VERSION                0x8000FFFF // Softwareversion der Slotfirmware

#define MB_SLOT_BUFFER_SIZE_SMALL      45  // Paketpuffergroesse bei FWVersion < 1.31
#define MB_SLOT_BUFFER_SIZE_LARGE      150 // Paketpuffergroesse bei FWVersion >= 1.31
#define MB_SLOT_BUFFER_DATA_SIZE_SMALL	(MB_SLOT_BUFFER_SIZE_SMALL - 13)
#define MB_SLOT_BUFFER_DATA_SIZE_LARGE	(MB_SLOT_BUFFER_SIZE_LARGE - 13)
//#############################################################################

// Versenden und Empfangen von Datenpacketen Tracen
//#define TRACE_DOCKING 1

// Datenstrucktur mit Daten zu einem Slot.
class TimeboySlot  
{
public:
    TimeboySlot()
    {
        Reset();
    }

public:
    void Reset()
    {
        // Member initialisieren
        _version = 0;
        _detectionState = 0;
        _ledState = 0;
        _ledStamp = 0;
        _receiveStampSlot = 0;
        _receiveStampTimeboy = 0;
    }
private:
    unsigned short _version;          // Ermittelte Firmwareversion
public:
    void setVersion(unsigned short version)	{ _version = version; }
    unsigned short getVersion() { return _version; }
private:
    unsigned char _detectionState;    // Ermittlung der Komperatorschaltung; 0 -> nicht ermittelt, 1 -> nicht verfuegbar, 2 -> verfuegbar
public:
    void setDetectionState(unsigned char detectionState) { _detectionState = detectionState; }
    unsigned char getDetectionState() { return _detectionState; }
private:
    unsigned char _ledState;          // Zuletzt gesetzten LED-Zustand: 0x00 -> NA, 0x01 -> ON, 0xFF -> OFF
public:
    void setLEDState(unsigned char ledState) { _ledState = ledState; }
    unsigned char getLEDState() { return _ledState; }
private:
    clock_t _ledStamp;                // clock() des letzten Zustand setzen.
public:
    void setLEDStamp(clock_t ledStamp) { _ledStamp = ledStamp; }
    clock_t getLEDStamp() { return _ledStamp; }
private:
    clock_t _receiveStampSlot;        // clock() des zuletzt gueltig empfangenen Pakets.
public:
    void setReceiveStampSlot(clock_t receiveStampSlot) { _receiveStampSlot = receiveStampSlot; }
    clock_t getReceiveStampSlot() { return _receiveStampSlot; }
private:
    clock_t _receiveStampTimeboy;     // clock() des zuletzt gueltig empfangene Antwort vom Timeboy
public:
    void setReceiveStampTimeboy(clock_t receiveStampTimeboy) { _receiveStampTimeboy = receiveStampTimeboy; }
    clock_t getReceiveStampTimeboy() { return _receiveStampTimeboy; }

};

// Automatische Jobs des Kommunikationsworkers
#define TSF_AUTO_LED         Bit1 // Ausschalten der Slot-LED durch Worker, nach vorgegebenem Timeout
#define TSF_AUTO_CLOSE       Bit2 // Schliessen der Schnittstelle, nach Abarbeitung aller Aufgaben.

// Klasse fuer die Verwaltung von Slots
class TimeboyStation
{
public:
    TimeboyStation()
    {
        // Member initialisiseren.
        _channelBase = NULL;

        // Verbindungsthread
        _communicationThread = 0;
        _threadState = end;
        _threadCommand = none;

        // Anzahl der Suspendanforderungen halten
        _suspendCounter = 0;

        // Flags fuer Steuerung der Idlezeit
        _flags = 0;
    }
    virtual ~TimeboyStation();

public:
    // Setzt die Basisklasse fuer die Kommunikationsschicht
    void setChannelBase(Channel4Base& channelBase) { _channelBase = &channelBase; }
    Channel4Base& getChannelBase() 
    { 
        if (_channelBase == NULL) throw; 
        return (Channel4Base&)*_channelBase; 
    }

    inline int isSlot(short deviceID)
    {
        return (getSlotFirmwareVersion(deviceID) > 0);
    }

    unsigned char getBufferSize(short deviceId)
    {
        // Bereich pruefen
        if (deviceId < 1 && deviceId > 255)
        {
            return 0;
        }

        // Je nach enthaltener Version
        switch(_timeboySlot[deviceId].getVersion()) 
        {
        case 0xFFFF: // Pruefung bereits durchgefuehrt und war nicht erfolgreich.
            break;

        case 0: // Ermittlung noch nicht durchgefuehrt
            return MB_SLOT_BUFFER_SIZE_SMALL;

        default:
            if (_timeboySlot[deviceId].getVersion() < 0x0131)
            {
                return MB_SLOT_BUFFER_SIZE_SMALL;
            }
            else if (_timeboySlot[deviceId].getVersion() >= 0x0131)
            {
                return MB_SLOT_BUFFER_SIZE_LARGE;
            }
        }

        return 0;
    }
    unsigned char getDataBufferSize(short deviceId)
    {
        // Bereich pruefen
        if (deviceId < 1 && deviceId > 255)
        {
            return 0;
        }

        switch(_timeboySlot[deviceId].getVersion()) 
        {
        case 0xFFFF: // Pruefung bereits durchgefuehrt und war nicht erfolgreich.
            break;

        case 0: // Ermittlung noch nicht durchgefuehrt
            return MB_SLOT_BUFFER_DATA_SIZE_SMALL;

        default:
            if (_timeboySlot[deviceId].getVersion() < 0x0131)
            {
                return MB_SLOT_BUFFER_DATA_SIZE_SMALL;
            }
            else if (_timeboySlot[deviceId].getVersion() >= 0x0131)
            {
                return MB_SLOT_BUFFER_DATA_SIZE_LARGE;
            }
        }

        return 0;
    }

    // Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:
#ifdef _MSC_VER
    static unsigned long WINAPI communicationThread(LPVOID lpParam);
#else
    static void *communicationThread(void *lpParam);
#endif
#ifdef _MSC_VER
    HANDLE _communicationThread;
    DWORD _threadID;
#else
    pthread_t _communicationThread;
#endif
protected:
    // Aktueller interner Zustand des Threads halten
    volatile enum threadState 
    {
        begin, // Bei Einstieg in Main-Routine
        end,   // Bei Ausstieg aus Main-Routine
        idle,  // Thread befindet sich im Leerlauf (laufende Pruefung auf anstehende Aufgaben)
        executeCommand, // Abarbeitung eines gesetzten Kommandos (threadCommand != none)
        dummy
    } _threadState;
public:
    threadState getThreadState() { return _threadState; }
protected:
    // Auszufuehrendes Kommando
    volatile enum threadCommand
    {
        none,  // Aktuell kein abzuarbeitender Befehl
        exit,  // Beendigung des Threads erfordert
        write, // Schreiben von Bytes erfordert (Daten in _commandParam)
        read   // Lesen von Bytes erfordert (Daten in _commandParam)
    } _threadCommand;
public:
    threadCommand getThreadCommand() { return _threadCommand; }
    int setThreadCommand(threadCommand tc)
    {
        if (_threadState == idle)
        {
            _threadCommand = tc;
            return 1;
        }

        return 0;
    }
    volatile struct  
    {
        int res;
        short deviceID;
        unsigned int baseAddress;
        int offset;
        unsigned char *buf;
        int count;
        int trials;
    } _commandParam;
    volatile int _suspendCounter;
protected:
    int startCommunicationThread();
    void stopCommunicationThread();

    // Gibt die FWVersion des Dockingslots an
    unsigned short getSlotFirmwareVersion(short deviceID);
public:
    void resetSlot(short deviceID);
    void resetSlots();

public: // Routinen fuer die Kommunikation mit Docking
    int writeBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials = IV_WRITE_TRIALS);
    int readBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials = IV_READ_TRIALS);
protected:
    int threadWriteBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int transmitTrials = IV_WRITE_TRIALS);
    int threadReadBytes(short deviceID, unsigned int baseAddress, int offset, unsigned char *buf, int count, int receiveTrials = IV_READ_TRIALS);

public:
    int getTimeboyModel(short deviceID, int &model);
    int setTimeboySlotState(short deviceID, int state, int commandResult = IV_ERROR_SUCCESS);

protected:
    int callDocking(unsigned char *txBuf, unsigned char txCount, unsigned char *rxBuf, unsigned char *rxCount);
    int setDockingLED(short deviceID, int state);

protected: // Routinen fuer die Flagverwaltung
    volatile unsigned int _flags;
public:
    enum FlagBit
    {
        Bit1  = 0x00000001, Bit2  = 0x00000002, Bit3  = 0x00000004, Bit4  = 0x00000008, Bit5  = 0x00000010, Bit6  = 0x00000020, Bit7  = 0x00000040, Bit8  = 0x00000080,
        Bit9  = 0x00000100, Bit10 = 0x00000200, Bit11 = 0x00000400, Bit12 = 0x00000800, Bit13 = 0x00001000, Bit14 = 0x00002000, Bit15 = 0x00004000, Bit16 = 0x00008000,
        Bit17 = 0x00010000, Bit18 = 0x00020000, Bit19 = 0x00040000, Bit20 = 0x00080000, Bit21 = 0x00100000, Bit22 = 0x00200000, Bit23 = 0x00400000, Bit24 = 0x00800000,
        Bit25 = 0x01000000, Bit26 = 0x02000000, Bit27 = 0x04000000, Bit28 = 0x08000000, Bit29 = 0x10000000, Bit30 = 0x20000000, Bit31 = 0x40000000, Bit32 = 0x80000000
    };
    void addFlag(FlagBit flag)
    {
        _flags |= flag;
    }
    void delFlag(FlagBit flag)
    {
        _flags &= ~flag;
    }
    int isFlag(FlagBit flag)
    {
        return ((_flags & flag) != 0);
    }

protected:
    int _autoLEDTimeout;
public:
    void setLEDAutoControl(int enable, int timeout)
    {
        _autoLEDTimeout = timeout;
        if (enable)
        {
            addFlag(TSF_AUTO_LED);
        }
        else
        {
            delFlag(TSF_AUTO_LED);
        }
    }

protected:
    Channel4Base *_channelBase;
    TimeboySlot _timeboySlot[255];

    // Aktuell bearbeitete DeviceID halten
protected:
    short _deviceIdRemote;
    inline void setDeviceIdRemote(short deviceID)
    {
        _deviceIdRemote = deviceID;
    }
    inline short getDeviceIdRemote()
    {
        return _deviceIdRemote;
    }

    // Helper zur Paketerstellung
protected:
    int binToUIntMSBFirst(unsigned char *Buf)
    {
        return int(Buf[0] << 8) | int(Buf[1]);
    }
    unsigned int binToULongMSBFirst(unsigned char *Dat)
    {
        unsigned int Val;

        Val  = Dat[0]; Val <<= 8;
        Val |= Dat[1]; Val <<= 8;
        Val |= Dat[2]; Val <<= 8;
        Val |= Dat[3];
        return Val;
    }

    unsigned short crc16(unsigned char *data, unsigned int size, unsigned short crc = 0xFFFF);
};

#endif
