/*###########################################################*/
/** \file aescryptsession.h
Autor                 : Bernd Ottmann             01/2012
*/  
/*###########################################################*/

#ifndef _AESCRYPTSESSION_H_
#define _AESCRYPTSESSION_H_

#include "../Common/aescrypt.h"
#include "../Channel/Channel4Protocol.h"
#include <memory.h>

// Klasse zur Verwaltung einer Session mittels AES.
class AESCryptSession
{
public:
    AESCryptSession()
    {
        // Verschluesselung verwalten
        m_cryptMode = 0;
        m_sessionTime = 0;
        m_sessionTimeout = 0;

        memset(m_sessionKey, 0, sizeof(m_sessionKey));
        m_sessionKeySize = 0;

        memset(m_rawCommunicationPassword, 0, sizeof(m_rawCommunicationPassword));
        m_rawCommunicationPasswordSize = 0;
        m_globalCommunicationPassword = 1; // Das Passwort, kann global veraendert werden.
    }

public:
    inline int IsGlobalCommunicationPassword() {
        return m_globalCommunicationPassword;
    }
    void ResetSessionKey() 
    {
        m_cryptMode = 0;
    }
    // Setzt das Standardpasswort in m_sessionKey
    inline void LoadDefaultSessionKey() {
        unsigned char sessionKey[16] = {
            0xc5, 0xc2, 0xc3, 0xc0, 0xc1, 0xd9, 0xdb, 0xf0, 
            0xcd, 0x07, 0xa7, 0xaf, 0xaa, 0x0a, 0x50, 0xae
        };
        if (m_cryptMode == IV_PACKET_CRYPT_MODE_PASSPHRASE) {

            memcpy(m_sessionKey, m_rawCommunicationPassword, m_rawCommunicationPasswordSize);
            m_sessionKeySize = m_rawCommunicationPasswordSize;

        } else {

            memcpy(m_sessionKey, sessionKey, sizeof(m_sessionKey));
            m_sessionKeySize = sizeof(m_sessionKey);
        }
    }
    inline bool IsValidUserKey() {
        unsigned char size = m_rawCommunicationPasswordSize;
        if (size == 0) {
            // strlen
            for (;size < 16 && m_rawCommunicationPassword[size]; size++);
            if (size == 0) return false; // Es liegt kein Passwort vor.
        }

        return true;
    }
    void InitCryptMode()
    {
        if (m_cryptMode == IV_PACKET_CRYPT_MODE_NO) {

            // Die verschluesselte Kommunikation wird zwar unterstuetzt,
            // soll aber nicht genutzt werden.

        } else if (m_cryptMode == IV_PACKET_CRYPT_MODE_DEFAULT || m_cryptMode == IV_PACKET_CRYPT_MODE_PASSPHRASE) {

            // Liegt kein Session-Key vor?
            if (m_sessionTime == 0) {
                LoadDefaultSessionKey();
            }
            m_aesTransmit.SetKey(m_sessionKey, m_sessionKeySize);
            m_aesReceive.SetKey(m_sessionKey, m_sessionKeySize);

        } else {

            // Keine Unterstuetzung der verschluesselten Kommunikation.
        }
    }

    int SetCommunicationPassword(unsigned char *password, int size, int sessionTimeout, int global) 
    {
        if (size == 0) {

            // strlen
            for (;size < 16 && password[size]; size++);
        } else if (size < 0 || size > 16) {

            return -IV_ERROR_FUNCTION_PARAMETER;
        }

        int different = 0;
        // Identisches Passwort und auch sessionTimeout sowie global gleich?
        if (memcmp(m_rawCommunicationPassword, password, size) != 0) {

            different++;
        }
        if (m_rawCommunicationPasswordSize != size) {

            different++;
        }
        if (m_sessionTimeout != sessionTimeout) {

            different++;
        }
        if (m_globalCommunicationPassword != global) {
            // Vorherige Werte sind gleich, dann wird das Flag uebernommen und ok.
            if (different == 0) {

                m_globalCommunicationPassword = global;
                return IV_ERROR_SUCCESS;
            }

            different++;
        }

        // Liegen unterschiede vor?
        if (different) {

            // Aktuellen Mode zurcksetzen.
            ResetSessionKey();

            // Werte uebernehmen
            memset(m_rawCommunicationPassword, 0, sizeof(m_rawCommunicationPassword));
            memcpy(m_rawCommunicationPassword, password, size);
            m_rawCommunicationPasswordSize = size;

            m_sessionTimeout = sessionTimeout;

            // Halten ob das Passwort global oder speziell fuer dieses Geraet gesetzt wurde.
            m_globalCommunicationPassword = global;
        }

        return IV_ERROR_SUCCESS;
    }

public:
    unsigned char m_cryptMode;
    unsigned char m_rawCommunicationPassword[16];
    unsigned char m_rawCommunicationPasswordSize;
    unsigned char m_sessionKey[16];
    unsigned char m_sessionKeySize;
    int m_sessionTimeout;
    int m_globalCommunicationPassword;
    time_t m_sessionTime;

public:
    unsigned short TransmitEncrypt(unsigned char *data, unsigned short raw_size)
    {
        return m_aesTransmit.Encrypt(data, raw_size);
    }
    unsigned int TransmitEncryptStream(unsigned char *data, unsigned int raw_size)
    {
        return m_aesTransmit.EncryptStream(data, raw_size);
    }
    unsigned char TransmitDecrypt(unsigned char *data, unsigned short *size)
    {
        return m_aesTransmit.Decrypt(data, size);
    }
    unsigned char TransmitDecryptStream(unsigned char *data, unsigned int *size)
    {
        return m_aesTransmit.DecryptStream(data, size);
    }

    unsigned short ReceiveEncrypt(unsigned char *data, unsigned short raw_size)
    {
        return m_aesReceive.Encrypt(data, raw_size);
    }
    unsigned int ReceiveEncryptStream(unsigned char *data, unsigned int raw_size)
    {
        return m_aesReceive.EncryptStream(data, raw_size);
    }
    unsigned char ReceiveDecrypt(unsigned char *data, unsigned short *size)
    {
        return m_aesReceive.Decrypt(data, size);
    }
    unsigned char ReceiveDecryptStream(unsigned char *data, unsigned int *size)
    {
        return m_aesReceive.DecryptStream(data, size);
    }

private:
    AESCrypt m_aesTransmit;
    AESCrypt m_aesReceive;
};

#endif
